@@ -40,18 +40,18 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.5.4.21.js';
+gTestfile = '13.5.4.20.js';
 
-START("13.5.4.21 - XMLList toXMLString()");
+START("13.5.4.20 - XMLList toString()");
 
-TEST(1, true, XMLList.prototype.hasOwnProperty("toXMLString"));
+TEST(1, true, XMLList.prototype.hasOwnProperty("toString"));
    
 x = <><alpha>one</alpha></>;
 
-TEST(2, "<alpha>one</alpha>", x.toXMLString());
+TEST(2, "one", x.toString());
 
 x = <><alpha>one</alpha><bravo>two</bravo></>;
 
-TEST(3, "<alpha>one</alpha>\n<bravo>two</bravo>", x.toXMLString());
+TEST(3, "<alpha>one</alpha>\n<bravo>two</bravo>", x.toString());
 
 END();
\ No newline at end of file
