@@ -16,7 +16,7 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Igor Bukanov
@@ -35,26 +35,57 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-310425-01.js';
+var gTestfile = 'regress-360681-02.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 310425;
-var summary = 'Array.indexOf/lastIndexOf edge cases';
+var BUGNUMBER = 360681;
+var summary = 'Regression from bug 224128';
 var actual = '';
 var expect = '';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
+
+function test()
+{
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
  
-expect = -1;
-actual = [].lastIndexOf(undefined, -1);
-reportCompare(expect, actual, summary);
+  expect = actual = 'No Crash';
 
-expect = -1;
-actual = [].indexOf(undefined, -1);
-reportCompare(expect, actual, summary);
+  var N = 1000;
 
-var x = [];
-x[1 << 30] = 1;
-expect = (1 << 30);
-actual = x.lastIndexOf(1);
-reportCompare(expect, actual, summary);
+// Make an array with a hole at the end
+  var a = Array(N);
+  for (i = 0; i < N - 1; ++i)
+    a[i] = 1;
+
+// array_sort due for array with N elements with allocates a temporary vector
+// with 2*N. Lets create strings that on 32 and 64 bit CPU cause allocation
+// of the same amount of memory + 1 word for their char arrays. After we GC
+// strings with a reasonable malloc implementation that memory will be most
+// likely reused in array_sort for the temporary vector. Then the bug causes
+// accessing the one-beyond-the-aloocation word and re-interpretation of
+// 0xFFF0FFF0 as GC thing.
+
+  var str1 = Array(2*(2*N + 1) + 1).join(String.fromCharCode(0xFFF0));
+  var str2 = Array(4*(2*N + 1) + 1).join(String.fromCharCode(0xFFF0));
+  gc();
+  str1 = str2 = null;
+  gc();
+
+  var firstCall = true;
+  a.sort(function (a, b) {
+	   if (firstCall) {
+	     firstCall = false;
+	     gc();
+	   }
+	   return a - b;
+	 });
+
+  reportCompare(expect, actual, summary);
+
+  exitFunc ('test');
+}
