@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Jeff Walden
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-351102-04.js';
+var gTestfile = 'regress-466905-04.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 351102;
-var summary = 'try/catch-guard/finally GC issues';
+var BUGNUMBER = 466905;
+var summary = 'Prototypes of sandboxed arrays';
 var actual = '';
 var expect = '';
 
@@ -52,16 +52,31 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  var f;
-  try
+
+  if (typeof evalcx != 'function')
   {
-    try { @foo } catch([] if gc()) { }
+    expect = actual = 'Test skipped: requires evalcx support';
   }
-  catch(ex)
+  else
   {
+    expect = true;
+
+    function createArray()
+    {
+      var a;                
+      for (var i = 0; i < 10; i++)
+        a = [1, 2, 3, 4, 5];
+      return a;
+    }
+
+    var sandbox = evalcx("lazy");
+    sandbox.createArray = createArray;
+    var p1 = Object.getPrototypeOf(createArray());
+    var p2 = Object.getPrototypeOf(evalcx("createArray()", sandbox));
+    print(actual = (p1 === p2));
   }
-  reportCompare(expect, actual, summary + ': 4');
+
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
