@@ -62,9 +62,6 @@
 #endif
 
 
-NS_DEFINE_IID(kInlineFrameCID, NS_INLINE_FRAME_CID);
-
-
 //////////////////////////////////////////////////////////////////////
 
 // Basic nsInlineFrame methods
@@ -75,18 +72,11 @@ NS_NewInlineFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
   return new (aPresShell) nsInlineFrame(aContext);
 }
 
-NS_IMETHODIMP
-nsInlineFrame::QueryInterface(const nsIID& aIID, void** aInstancePtr)
-{
-  NS_PRECONDITION(aInstancePtr, "null out param");
+NS_IMPL_FRAMEARENA_HELPERS(nsInlineFrame)
 
-  if (aIID.Equals(kInlineFrameCID)) {
-    *aInstancePtr = this;
-    return NS_OK;
-  }
-
-  return nsInlineFrameSuper::QueryInterface(aIID, aInstancePtr);
-}
+NS_QUERYFRAME_HEAD(nsInlineFrame)
+  NS_QUERYFRAME_ENTRY(nsInlineFrame)
+NS_QUERYFRAME_TAIL_INHERITING(nsInlineFrameSuper)
 
 #ifdef DEBUG
 NS_IMETHODIMP
@@ -133,16 +123,39 @@ nsInlineFrame::IsSelfEmpty()
   // XXX Top and bottom removed, since they shouldn't affect things, but this
   // doesn't really match with nsLineLayout.cpp's setting of
   // ZeroEffectiveSpanBox, anymore, so what should this really be?
-  if (border->GetActualBorderWidth(NS_SIDE_RIGHT) != 0 ||
-      border->GetActualBorderWidth(NS_SIDE_LEFT) != 0 ||
-      !IsPaddingZero(padding->mPadding.GetRightUnit(),
-                     padding->mPadding.GetRight()) ||
-      !IsPaddingZero(padding->mPadding.GetLeftUnit(),
-                     padding->mPadding.GetLeft()) ||
-      !IsMarginZero(margin->mMargin.GetRightUnit(),
-                    margin->mMargin.GetRight()) ||
-      !IsMarginZero(margin->mMargin.GetLeftUnit(),
-                    margin->mMargin.GetLeft())) {
+  PRBool haveRight =
+    border->GetActualBorderWidth(NS_SIDE_RIGHT) != 0 ||
+    !IsPaddingZero(padding->mPadding.GetRightUnit(),
+                   padding->mPadding.GetRight()) ||
+    !IsMarginZero(margin->mMargin.GetRightUnit(),
+                  margin->mMargin.GetRight());
+  PRBool haveLeft =
+    border->GetActualBorderWidth(NS_SIDE_LEFT) != 0 ||
+    !IsPaddingZero(padding->mPadding.GetLeftUnit(),
+                   padding->mPadding.GetLeft()) ||
+    !IsMarginZero(margin->mMargin.GetLeftUnit(),
+                  margin->mMargin.GetLeft());
+  if (haveLeft || haveRight) {
+    if (GetStateBits() & NS_FRAME_IS_SPECIAL) {
+      PRBool haveStart, haveEnd;
+      if (NS_STYLE_DIRECTION_LTR == GetStyleVisibility()->mDirection) {
+        haveStart = haveLeft;
+        haveEnd = haveRight;
+      } else {
+        haveStart = haveRight;
+        haveEnd = haveLeft;
+      }
+      // For special frames, ignore things we know we'll skip in GetSkipSides.
+      // XXXbz should we be doing this for non-special frames too, in a more
+      // general way?
+
+      // Get the first continuation eagerly, as a performance optimization, to
+      // avoid having to get it twice..
+      nsIFrame* firstCont = GetFirstContinuation();
+      return
+        (!haveStart || nsLayoutUtils::FrameIsInLastPartOfIBSplit(firstCont)) &&
+        (!haveEnd || nsLayoutUtils::FrameIsInFirstPartOfIBSplit(firstCont));
+    }
     return PR_FALSE;
   }
   return PR_TRUE;
@@ -238,6 +251,8 @@ nsInlineFrame::ReparentFloatsForInlineChild(nsIFrame* aOurLineContainer,
                                             nsIFrame* aFrame,
                                             PRBool aReparentSiblings)
 {
+  // XXXbz this would be better if it took a nsFrameList or a frame
+  // list slice....
   NS_ASSERTION(aOurLineContainer->GetNextContinuation() ||
                aOurLineContainer->GetPrevContinuation(),
                "Don't call this when we have no continuation, it's a waste");
@@ -306,12 +321,13 @@ nsInlineFrame::Reflow(nsPresContext*          aPresContext,
    // Check for an overflow list with our prev-in-flow
   nsInlineFrame* prevInFlow = (nsInlineFrame*)GetPrevInFlow();
   if (nsnull != prevInFlow) {
-    nsIFrame* prevOverflowFrames = prevInFlow->GetOverflowFrames(aPresContext, PR_TRUE);
+    nsAutoPtr<nsFrameList> prevOverflowFrames(prevInFlow->StealOverflowFrames());
 
     if (prevOverflowFrames) {
       // When pushing and pulling frames we need to check for whether any
       // views need to be reparented.
-      nsHTMLContainerFrame::ReparentFrameViewList(aPresContext, prevOverflowFrames,
+      nsHTMLContainerFrame::ReparentFrameViewList(aPresContext,
+                                                  *prevOverflowFrames,
                                                   prevInFlow, this);
 
       // Check if we should do the lazilySetParentPointer optimization.
@@ -321,22 +337,22 @@ nsInlineFrame::Reflow(nsPresContext*          aPresContext,
       // will be taken from prevOverflowFrames.
       if ((GetStateBits() & NS_FRAME_FIRST_REFLOW) && mFrames.IsEmpty() &&
           !GetNextInFlow()) {
-        // If our child list is empty, just set the child list rather than
-        // calling InsertFrame(). This avoids having to get the last child
-        // frame in the list.
+        // If our child list is empty, just put the new frames into it.
         // Note that we don't set the parent pointer for the new frames. Instead wait
         // to do this until we actually reflow the frame. If the overflow list contains
         // thousands of frames this is a big performance issue (see bug #5588)
-        mFrames.SetFrames(prevOverflowFrames);
+        mFrames.SetFrames(*prevOverflowFrames);
         lazilySetParentPointer = PR_TRUE;
       } else {
         // Assign all floats to our block if necessary
         if (lineContainer && lineContainer->GetPrevContinuation()) {
-          ReparentFloatsForInlineChild(lineContainer, prevOverflowFrames, PR_TRUE);
+          ReparentFloatsForInlineChild(lineContainer,
+                                       prevOverflowFrames->FirstChild(),
+                                       PR_TRUE);
         }
         // Insert the new frames at the beginning of the child list
         // and set their parent pointer
-        mFrames.InsertFrames(this, nsnull, prevOverflowFrames);
+        mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
       }
     }
   }
@@ -347,19 +363,20 @@ nsInlineFrame::Reflow(nsPresContext*          aPresContext,
     // If it's our initial reflow, then we should not have an overflow list.
     // However, add an assertion in case we get reflowed more than once with
     // the initial reflow reason
-    nsIFrame* overflowFrames = GetOverflowFrames(aPresContext, PR_FALSE);
-    NS_ASSERTION(!overflowFrames, "overflow list is not empty for initial reflow");
+    nsFrameList* overflowFrames = GetOverflowFrames();
+    NS_ASSERTION(!overflowFrames || overflowFrames->IsEmpty(),
+                 "overflow list is not empty for initial reflow");
   }
 #endif
   if (!(GetStateBits() & NS_FRAME_FIRST_REFLOW)) {
-    nsIFrame* overflowFrames = GetOverflowFrames(aPresContext, PR_TRUE);
+    nsAutoPtr<nsFrameList> overflowFrames(StealOverflowFrames());
     if (overflowFrames) {
       NS_ASSERTION(mFrames.NotEmpty(), "overflow list w/o frames");
 
       // Because we lazily set the parent pointer of child frames we get from
       // our prev-in-flow's overflow list, it's possible that we have not set
       // the parent pointer for these frames.
-      mFrames.AppendFrames(this, overflowFrames);
+      mFrames.AppendFrames(this, *overflowFrames);
     }
   }
 
@@ -396,7 +413,7 @@ nsInlineFrame::Reflow(nsPresContext*          aPresContext,
   rv = ReflowFrames(aPresContext, aReflowState, irs, aMetrics, aStatus);
   
   // Note: the line layout code will properly compute our
-  // NS_FRAME_OUTSIDE_CHILDREN state for us.
+  // overflow-rect state for us.
 
   NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aMetrics);
   return rv;
@@ -409,6 +426,22 @@ nsInlineFrame::CanContinueTextRun() const
   return PR_TRUE;
 }
 
+/* virtual */ void
+nsInlineFrame::PullOverflowsFromPrevInFlow()
+{
+  nsInlineFrame* prevInFlow = static_cast<nsInlineFrame*>(GetPrevInFlow());
+  if (prevInFlow) {
+    nsAutoPtr<nsFrameList> prevOverflowFrames(prevInFlow->StealOverflowFrames());
+    if (prevOverflowFrames) {
+      // Assume that our prev-in-flow has the same line container that we do.
+      nsHTMLContainerFrame::ReparentFrameViewList(PresContext(),
+                                                  *prevOverflowFrames,
+                                                  prevInFlow, this);
+      mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
+    }
+  }
+}
+
 nsresult
 nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
                             const nsHTMLReflowState& aReflowState,
@@ -422,7 +455,10 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
   nsLineLayout* lineLayout = aReflowState.mLineLayout;
   PRBool ltr = (NS_STYLE_DIRECTION_LTR == aReflowState.mStyleVisibility->mDirection);
   nscoord leftEdge = 0;
-  if (nsnull == GetPrevContinuation()) {
+  // Don't offset by our start borderpadding if we have a prev continuation or
+  // if we're in the last part of an {ib} split.
+  if (!GetPrevContinuation() &&
+      !nsLayoutUtils::FrameIsInLastPartOfIBSplit(this)) {
     leftEdge = ltr ? aReflowState.mComputedBorderPadding.left
                    : aReflowState.mComputedBorderPadding.right;
   }
@@ -555,8 +591,7 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
 #ifdef DEBUG
   if (NS_FRAME_IS_COMPLETE(aStatus)) {
     // We can't be complete AND have overflow frames!
-    nsIFrame* overflowFrames = GetOverflowFrames(aPresContext, PR_FALSE);
-    NS_ASSERTION(!overflowFrames, "whoops");
+    NS_ASSERTION(!GetOverflowFrames(), "whoops");
   }
 #endif
 
@@ -569,12 +604,25 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
   // whitespace in an inline element don't affect the line-height.
   aMetrics.width = lineLayout->EndSpan(this);
 
-  // Compute final width
-  if (nsnull == GetPrevContinuation()) {
+  // Compute final width.
+
+  // Make sure to not include our start border and padding if we have a prev
+  // continuation or if we're in the last part of an {ib} split.
+  if (!GetPrevContinuation() &&
+      !nsLayoutUtils::FrameIsInLastPartOfIBSplit(this)) {
     aMetrics.width += ltr ? aReflowState.mComputedBorderPadding.left
                           : aReflowState.mComputedBorderPadding.right;
   }
-  if (NS_FRAME_IS_COMPLETE(aStatus) && (!GetNextContinuation() || GetNextInFlow())) {
+
+  /*
+   * We want to only apply the end border and padding if we're the last
+   * continuation and not in the first part of an {ib} split.  To be the last
+   * continuation we have to be complete (so that we won't get a next-in-flow)
+   * and have no non-fluid continuations on our continuation chain.
+   */
+  if (NS_FRAME_IS_COMPLETE(aStatus) &&
+      !GetLastInFlow()->GetNextContinuation() &&
+      !nsLayoutUtils::FrameIsInFirstPartOfIBSplit(this)) {
     aMetrics.width += ltr ? aReflowState.mComputedBorderPadding.right
                           : aReflowState.mComputedBorderPadding.left;
   }
@@ -803,6 +851,28 @@ nsInlineFrame::GetSkipSides() const
       // edge border render.
     }
   }
+
+  if (GetStateBits() & NS_FRAME_IS_SPECIAL) {
+    // The first part of an {ib} split should always skip the "end" side (as
+    // determined by this frame's direction) and the last part of such a split
+    // should alwas skip the "start" side.  But figuring out which part of the
+    // split we are involves getting our first continuation, which might be
+    // expensive.  So don't bother if we already have the relevant bits set.
+    PRBool ltr = (NS_STYLE_DIRECTION_LTR == GetStyleVisibility()->mDirection);
+    PRIntn startBit = (1 << (ltr ? NS_SIDE_LEFT : NS_SIDE_RIGHT));
+    PRIntn endBit = (1 << (ltr ? NS_SIDE_RIGHT : NS_SIDE_LEFT));
+    if (((startBit | endBit) & skip) != (startBit | endBit)) {
+      // We're missing one of the skip bits, so check whether we need to set it.
+      if (nsLayoutUtils::FrameIsInFirstPartOfIBSplit(this)) {
+        skip |= endBit;
+      } else {
+        NS_ASSERTION(nsLayoutUtils::FrameIsInLastPartOfIBSplit(this),
+                     "How did that happen?");
+        skip |= startBit;
+      }
+    }
+  }
+
   return skip;
 }
 
@@ -837,15 +907,14 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
 
 static void
 ReParentChildListStyle(nsPresContext* aPresContext,
-                       nsFrameList& aFrameList,
+                       const nsFrameList::Slice& aFrames,
                        nsIFrame* aParentFrame)
 {
   nsFrameManager *frameManager = aPresContext->FrameManager();
 
-  for (nsIFrame* kid = aFrameList.FirstChild(); kid;
-       kid = kid->GetNextSibling()) {
-    NS_ASSERTION(kid->GetParent() == aParentFrame, "Bogus parentage");
-    frameManager->ReParentStyleContext(kid);
+  for (nsFrameList::Enumerator e(aFrames); !e.AtEnd(); e.Next()) {
+    NS_ASSERTION(e.get()->GetParent() == aParentFrame, "Bogus parentage");
+    frameManager->ReParentStyleContext(e.get());
   }
 }
 
@@ -855,6 +924,8 @@ NS_NewFirstLineFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
   return new (aPresShell) nsFirstLineFrame(aContext);
 }
 
+NS_IMPL_FRAMEARENA_HELPERS(nsFirstLineFrame)
+
 #ifdef DEBUG
 NS_IMETHODIMP
 nsFirstLineFrame::GetFrameName(nsAString& aResult) const
@@ -910,27 +981,28 @@ nsFirstLineFrame::Reflow(nsPresContext* aPresContext,
   // Check for an overflow list with our prev-in-flow
   nsFirstLineFrame* prevInFlow = (nsFirstLineFrame*)GetPrevInFlow();
   if (nsnull != prevInFlow) {
-    nsIFrame* prevOverflowFrames = prevInFlow->GetOverflowFrames(aPresContext, PR_TRUE);
+    nsAutoPtr<nsFrameList> prevOverflowFrames(prevInFlow->StealOverflowFrames());
     if (prevOverflowFrames) {
-      nsFrameList frames(prevOverflowFrames);
-      
       // Assign all floats to our block if necessary
       if (lineContainer && lineContainer->GetPrevContinuation()) {
-        ReparentFloatsForInlineChild(lineContainer, prevOverflowFrames, PR_TRUE);
+        ReparentFloatsForInlineChild(lineContainer,
+                                     prevOverflowFrames->FirstChild(),
+                                     PR_TRUE);
       }
-      mFrames.InsertFrames(this, nsnull, prevOverflowFrames);
-      ReParentChildListStyle(aPresContext, frames, this);
+      const nsFrameList::Slice& newFrames =
+        mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
+      ReParentChildListStyle(aPresContext, newFrames, this);
     }
   }
 
   // It's also possible that we have an overflow list for ourselves
-  nsIFrame* overflowFrames = GetOverflowFrames(aPresContext, PR_TRUE);
+  nsAutoPtr<nsFrameList> overflowFrames(StealOverflowFrames());
   if (overflowFrames) {
     NS_ASSERTION(mFrames.NotEmpty(), "overflow list w/o frames");
-    nsFrameList frames(overflowFrames);
 
-    mFrames.AppendFrames(nsnull, overflowFrames);
-    ReParentChildListStyle(aPresContext, frames, this);
+    const nsFrameList::Slice& newFrames =
+      mFrames.AppendFrames(nsnull, *overflowFrames);
+    ReParentChildListStyle(aPresContext, newFrames, this);
   }
 
   // Set our own reflow state (additional state above and beyond
@@ -1009,6 +1081,21 @@ nsFirstLineFrame::Reflow(nsPresContext* aPresContext,
   return rv;
 }
 
+/* virtual */ void
+nsFirstLineFrame::PullOverflowsFromPrevInFlow()
+{
+  nsFirstLineFrame* prevInFlow = static_cast<nsFirstLineFrame*>(GetPrevInFlow());
+  if (prevInFlow) {
+    nsAutoPtr<nsFrameList> prevOverflowFrames(prevInFlow->StealOverflowFrames());
+    if (prevOverflowFrames) {
+      // Assume that our prev-in-flow has the same line container that we do.
+      const nsFrameList::Slice& newFrames =
+        mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
+      ReParentChildListStyle(PresContext(), newFrames, this);
+    }
+  }
+}
+
 //////////////////////////////////////////////////////////////////////
 
 nsIFrame*
@@ -1017,6 +1104,8 @@ NS_NewPositionedInlineFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
   return new (aPresShell) nsPositionedInlineFrame(aContext);
 }
 
+NS_IMPL_FRAMEARENA_HELPERS(nsPositionedInlineFrame)
+
 void
 nsPositionedInlineFrame::Destroy()
 {
@@ -1026,7 +1115,7 @@ nsPositionedInlineFrame::Destroy()
 
 NS_IMETHODIMP
 nsPositionedInlineFrame::SetInitialChildList(nsIAtom*        aListName,
-                                             nsIFrame*       aChildList)
+                                             nsFrameList&    aChildList)
 {
   nsresult  rv;
 
@@ -1041,7 +1130,7 @@ nsPositionedInlineFrame::SetInitialChildList(nsIAtom*        aListName,
 
 NS_IMETHODIMP
 nsPositionedInlineFrame::AppendFrames(nsIAtom*        aListName,
-                                      nsIFrame*       aFrameList)
+                                      nsFrameList&    aFrameList)
 {
   nsresult  rv;
   
@@ -1057,7 +1146,7 @@ nsPositionedInlineFrame::AppendFrames(nsIAtom*        aListName,
 NS_IMETHODIMP
 nsPositionedInlineFrame::InsertFrames(nsIAtom*        aListName,
                                       nsIFrame*       aPrevFrame,
-                                      nsIFrame*       aFrameList)
+                                      nsFrameList&    aFrameList)
 {
   nsresult  rv;
 
@@ -1091,7 +1180,8 @@ nsPositionedInlineFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
                                           const nsRect&           aDirtyRect,
                                           const nsDisplayListSet& aLists)
 {
-  aBuilder->MarkFramesForDisplayList(this, mAbsoluteContainer.GetFirstChild(), aDirtyRect);
+  aBuilder->MarkFramesForDisplayList(this, mAbsoluteContainer.GetChildList(),
+				     aDirtyRect);
   return nsHTMLContainerFrame::BuildDisplayList(aBuilder, aDirtyRect, aLists);
 }
 
@@ -1104,13 +1194,13 @@ nsPositionedInlineFrame::GetAdditionalChildListName(PRInt32 aIndex) const
   return nsnull;
 }
 
-nsIFrame*
-nsPositionedInlineFrame::GetFirstChild(nsIAtom* aListName) const
+nsFrameList
+nsPositionedInlineFrame::GetChildList(nsIAtom* aListName) const
 {
   if (nsGkAtoms::absoluteList == aListName)
-    return mAbsoluteContainer.GetFirstChild();
+    return mAbsoluteContainer.GetChildList();
 
-  return nsInlineFrame::GetFirstChild(aListName);
+  return nsInlineFrame::GetChildList(aListName);
 }
 
 nsIAtom*
